@@ -1,5 +1,12 @@
-import tensorflow as tf
+tf = None
+try:
+  tf = __import__("tensorflow-gpu")
+  tf.operation_that_requires_gpu()
+except:
+  tf = __import__("tensorflow")
+  
 tf.compat.v1.logging.set_verbosity(tf.compat.v1.logging.ERROR)
+
 from os import environ
 environ['TF_CPP_MIN_LOG_LEVEL']='3'
 
@@ -8,49 +15,35 @@ import pickle
 import numpy as np
 from .models import *
 from mtcnn import MTCNN
-from sklearn.linear_model import LogisticRegression
 from tensorflow.keras.applications.imagenet_utils import preprocess_input
 
 class FaceRecognitionSystem(object):
     
-    """
-    ### Description
-        Face Recognition System creates an instance of the class with the following capabilities:
-        1. Face Detection with detectFaces, faceLocations, facialFeatures functions.
-        2. Face Features extraction with faceEmbeddings functions.
-        3. Face alignment for better face recognition.
-        4. Faces comparison with faceDistance and compareFaces.
-        5. Add face to database through camera
-        6. Add face to database from file (can use folder loop to add multiple faces at once)
-        
-        Face Recognition relies on two pickle files:
-        1. pickle file containing dictionary {id : name} of known faces.
-        2. pickle file containing dictionary {id : listOfEmbeddings} of known faces.
-        
-    """
     def __init__(self,
                  embedding_model=None,
                  weights=None,
                  face_classifier=None, 
                  **kwargs): 
+        
         """
         ### Description
-            1. Initializes MTCNN Face detection model.
-            2. Initializes FaceNet or VGGFace model to extract embeddings from face images.
-            3. Connects to Database through DatabaseConnection class.
-            4. assigns to instance variable a dictionary {id : name} of known faces from the database.
-            5. assigns to instance variable a dictionary {id : listOfEmbeddings} of known faces from the database.
-            6. assingns to instance variable the desired face size required for the embeddings model.
+            Face Recognition System creates an instance of the class with the following capabilities:
+            1. Calls MTCNN face detection model object.
+            2. Calls FaceNet or VGGFace model to extract embeddings (features) from face images.
+            3. Creates a connection to the database of known faces by calling the "Database" class object. 
+                - self.db is the dictionary of known faces with id : name key-value pairs.
+                - self.embeddings is the dictionary of known faces with id : embeddings key-value pairs.
+            4. Loads face classifier that was trained on the database of known faces.
 
-        ### Args:
-            'face_size' (int): face size required for the embeddings model e.g 160.
-            'names_pkl' (str): path to pickle file containing dictionary {id : name} of known faces.
-            'embeddings_pkl' (str):  path to pickle file containing dictionary {id : listOfEmbeddings} of known faces.
-            
+        ### Args
+            'embedding_model' (str): name of the desired feature extractor 'facenet' or 'vggface'. Defaults to 'facenet'.
+            'weights' (str): path to the weights of chosen embedding model.
+            'face_classifier' (str): path to the face classifier model.
+            **kwargs:
+                'db_filel' (str): path to pickle file containing dictionary {id : name} of known faces.
+                'embeddings_file' (str):  path to pickle file containing dictionary {id : listOfEmbeddings} of known faces.
         """
-        print("""
-**********************************
-Loading Face Recognition System...""")
+        print("Loading Face Recognition System...")
         
         self.detector = MTCNN()
         
@@ -58,7 +51,7 @@ Loading Face Recognition System...""")
             self.predictor, self.face_size = embeddingsPredictor(which=embedding_model, path=weights)
         
         if "db_file" in kwargs:
-            self.connection = DatabaseConnection(**kwargs)
+            self.connection = Database(**kwargs)
             self.db = self.connection.db
             self.embeddings = self.connection.embeddings
             
@@ -72,7 +65,7 @@ Loading Face Recognition System...""")
                       face_location=None, 
                       facial_features=None):
         """
-        This function take an image containing face, 
+        This function takes an image containing face, 
         aligns the face so that eyes are horizontal,
         and crops to given face size dimension.
 
@@ -115,9 +108,9 @@ Loading Face Recognition System...""")
         desired_face_height = face_size
 
         # determine the scale of the new resulting image by taking
-        # the ratio of the distance between eyes in the *current*
+        # the ratio of the distance between eyes in the current
         # image to the ratio of distance between eyes in the
-        # *desired* image
+        # desired image
         dist = np.sqrt((dX ** 2) + (dY ** 2))
         desiredDist = (desired_right_eye_x - desired_left_eye[0])
         desiredDist *= desired_face_width
@@ -205,12 +198,15 @@ Loading Face Recognition System...""")
     def detectFaces(self, image):
         
         """
-        ### Description:
+        ### Description
             Finds faces on a given image array and 
             returns list of location coordinates and 
             facial features coordinates.
+        
+        ### Args
+            image (ndarray) : image containing faces
 
-        ### Returns:
+        ### Returns
             (list): list of face location bounding box coordinates
             (list): list of facial features dictionaries
             
@@ -226,7 +222,8 @@ Loading Face Recognition System...""")
                      {   
                         ...
                      }
-                   ]```
+                   ]
+        ```
         """
         faces = self.detector.detect_faces(image)
         bboxes = [] # face locations coordinates
@@ -332,7 +329,7 @@ Loading Face Recognition System...""")
     
         return name    
     
-    def addEmbeddingsFromFile(self, filename, name):
+    def __addEmbeddingsFromFile(self, filename, name):
         """
         ### Description
             Adds new face embeddings containing extracted features from a given image. 
@@ -364,7 +361,7 @@ Loading Face Recognition System...""")
         else:
             print("No faces detected in the given image.")
     
-    def addEmbeddingsFromCamera(self, name):
+    def __addEmbeddingsFromCamera(self, name):
         """
         ### Description
             Using webcam adds new face embeddings containing extracted features to database. 
@@ -423,9 +420,39 @@ Loading Face Recognition System...""")
             if 'face_embedding' in locals():
                 self.connection.dumpEmbeddings()
 
+    def addFaceToDatabase(self, name, method="file"):
+	
+        if method not in ("file", "camera"):
+            raise AttributeError("Invalid value for method. Please use 'file' or 'camera'.")
+               
+        if method == "camera":
+            self.__addEmbeddingsFromCamera(name)
+
+        else:
+            import re
+            from tkinter import Tk
+            from tkinter.filedialog import askopenfilename
             
+            regex = re.compile('[0-9_\.\/_]+.jpg')
+            root = Tk()
+            root.withdraw()
+            filename = askopenfilename(title='Select Image file')
+            
+            self.__addEmbeddingsFromFile(filename, name)        
     
-class DatabaseConnection(object):
+    def addFacesUsingLoop(self, base):
+        
+        import os
+        for folder in os.listdir(base):
+            if folder[0] == ".":
+                continue
+
+            path = os.path.join(base, folder)
+            for image in os.listdir(path):
+                if image[0] == ".":
+                    continue
+                self.__addEmbeddingsFromFile(os.path.join(path, image), folder)
+class Database(object):
     """
     ### Description 
         This class creates a database object which has the following functionalities:
@@ -444,22 +471,19 @@ class DatabaseConnection(object):
         self.embeddings_file = embeddings_file
         try:
             with open(self.db_file, "rb") as f:
-                db = pickle.load(f)
+                self.db = pickle.load(f)
                 
             with open(self.embeddings_file, "rb") as f2:
-                embeddings = pickle.load(f2)  
+                self.embeddings = pickle.load(f2)  
         except:
             print("No db file exists. Creating new one")
-            db = {}
-            embeddings = {}
+            self.db = {}
+            self.embeddings = {}
             with open(self.db_file, "wb") as f:
-                pickle.dump(db)
+                pickle.dump(self.db, f)
             
             with open(self.embeddings_file, "wb") as f2:
-                pickle.dump(embeddings)
-            
-        self.db = db
-        self.embeddings = embeddings
+                pickle.dump(self.embeddings, f2)
         
     def dumpEmbeddings(self):
         """
